local Talented = Talented
local ipairs = ipairs

function Talented:IsTemplateAtCap(template)
	return self.db.profile.level_cap and self:GetPointCount(template) >= self.max_talent_points
end

function Talented:GetPointCount(template)
	local total = 0
	local info = self:UncompressSpellData(template.class)
	for tab in ipairs(info) do
		total = total + self:GetTalentTabCount(template, tab)
	end
	return total
end

function Talented:GetTalentTabCount(template, tab)
	local total = 0
	for _, value in ipairs(template[tab]) do
		total = total + value
	end
	return total
end

function Talented:ClearTalentTab(tab)
	local template = self.template
	if template and template ~= self.current and self.mode == "edit" then
		local tab = template[tab]
		for index, value in ipairs(tab) do
			tab[index] = 0
		end
	end
	self:UpdateView()
end

function Talented:GetTalentState(template, tab, index)
	local s
	local info = self:UncompressSpellData(template.class)[tab][index]
	local tier = (info.row - 1) * 5
	local count = self:GetTalentTabCount(template, tab)

	if count < tier then
		s = false
	else
		s = true
		if info.req and self:GetTalentState(template, tab, info.req) ~= "full" then
			s = false
		end
	end

	if not s then
		s = "unavailable"
	else
		local value = template[tab][index]
		if value == #info.ranks then
			s = "full"
		elseif value == 0 then
			s = "empty"
		else
			s = "available"
		end
	end
	return s
end

function Talented:UpdateTalent(tab, index, offset)
	if self.mode ~= "edit" then return end
	if self.template == self.current then
		-- Applying talent
		if offset > 0 then
			self:LearnTalent(tab, index)
		end
		return
	end
	local template = self.template

	if offset > 0 and self:IsTemplateAtCap(template) then return end
	local s = self:GetTalentState(template, tab, index)

	local ranks = self:GetTalentRanks(template.class, tab, index)
	local original = template[tab][index]
	local value = original + offset
	if value < 0 or s == "unavailable" then
		value = 0
	elseif value > ranks then
		value = ranks
	end
	self:Debug("Updating %d-%d : %d -> %d (%d)", tab, index, original, value, offset)
	if value == original or not self:ValidateTalentBranch(template, tab, index, value) then
		return
	end
	template[tab][index] = value
	self:UpdateView()
	self:UpdateTooltip()
	return true
end

function Talented:OnTalentClick(button, tab, index)
	if IsModifiedClick("CHATLINK") then
		local link = self:GetTalentLink(self.template, tab, index)
		if link then
			ChatEdit_InsertLink(link)
		end
	else
		self:UpdateTalent(tab, index, button == "LeftButton" and 1 or -1)
	end
end

function Talented:ValidateTalentBranch(template, tab, index, newvalue)
	local count = 0
	local tree = self:UncompressSpellData(template.class)[tab]
	local ttab = template[tab]
	for i, talent in ipairs(tree) do
		local value = i == index and newvalue or ttab[i]
		if value > 0 then
			local tier = (talent.row - 1) * 5
			if count < tier then
				self:Debug("Update refused because of tier")
				return false
			end
			local r = talent.req
			if r then
				local rvalue = r == index and newvalue or ttab[r]
				if rvalue < #tree[r].ranks then
					self:Debug("Update refused because of prereq")
					return false
				end
			end
			count = count + value
		end
	end
	return true
end

function Talented:ValidateTemplate(template)
	local class = template.class
	if not class then return end
	local info = self:UncompressSpellData(class)
	for tab, tree in ipairs(info) do
		local t = template[tab]
		local count = 0
		for i, talent in ipairs(tree) do
			local value = t[i]
			if value > 0 then
				if count < (talent.row - 1) * 5 then return end
				if value > #talent.ranks then return end
				local r = talent.req
				if r then
					if t[r] < #tree[r].ranks then return end
				end
				count = count + value
			end
		end
	end
	return true
end
